@@ -12,7 +12,7 @@ License:     MIT
 Maintainer:  Stefan Saasen <stefan@saasen.me>
 Stability:   experimental
 
-This implementation of JWT is based on <http://tools.ietf.org/html/draft-ietf-oauth-json-web-token-30> (Version 30)
+This implementation of JWT is based on <https://tools.ietf.org/html/rfc7519>
 but currently only implements the minimum required to work with the Atlassian Connect framework.
 
 Known limitations:
@@ -23,9 +23,6 @@ Known limitations:
    ('exp', 'nbf', 'iat').
 
    * Registered claims are not validated
-
-   * This implementation uses the term `JWTHeader` instead of `JOSEHeader` (changed in version 23 of the JWT draft).
-     Future versions will move to the offical term once that has stabilised.
 -}
 module Web.JWT
     (
@@ -43,6 +40,7 @@ module Web.JWT
     -- ** Common
     , tokenIssuer
     , secret
+    , binarySecret
     -- ** JWT structure
     , claims
     , header
@@ -50,6 +48,7 @@ module Web.JWT
     -- ** JWT claims set
     , auds
     , intDate
+    , numericDate
     , stringOrURI
     , stringOrURIToText
     , secondsSinceEpoch
@@ -69,20 +68,24 @@ module Web.JWT
     , JWTClaimsSet(..)
     , ClaimsMap
     , IntDate
+    , NumericDate
     , StringOrURI
     , JWTHeader
+    , JOSEHeader
 
     , module Data.Default
     ) where
 
 import qualified Data.ByteString.Lazy.Char8 as BL (fromStrict, toStrict)
+import qualified Data.ByteString.Extended as BS
 import qualified Data.Text.Extended         as T
 import qualified Data.Text.Encoding         as TE
 
 import           Control.Applicative
 import           Control.Monad
-import qualified Crypto.Hash.SHA256         as SHA
-import qualified Crypto.MAC.HMAC            as HMAC
+import           Crypto.Hash.Algorithms
+import           Crypto.MAC.HMAC
+import           Data.ByteArray.Encoding
 import           Data.Aeson                 hiding (decode, encode)
 import qualified Data.Aeson                 as JSON
 import           Data.Default
@@ -92,17 +95,24 @@ import           Data.Maybe
 import           Data.Scientific
 import           Data.Time.Clock            (NominalDiffTime)
 import qualified Network.URI                as URI
-import           Web.Base64
 import           Prelude                    hiding (exp)
 
+-- $setup
+-- The code examples in this module require GHC's `OverloadedStrings`
+-- extension:
+--
+-- >>> :set -XOverloadedStrings
 
 type JSON = T.Text
 
+{-# DEPRECATED JWTHeader "Use JOSEHeader instead. JWTHeader will be removed in 1.0" #-}
+type JWTHeader = JOSEHeader
+
 -- | The secret used for calculating the message signature
-newtype Secret = Secret T.Text
+newtype Secret = Secret BS.ByteString
 
 instance Eq Secret where
-    (Secret s1) == (Secret s2) = s1 `T.constTimeCompare` s2
+    (Secret s1) == (Secret s2) = s1 `BS.constTimeCompare` s2
 
 instance Show Secret where
     show _ = "<secret>"
@@ -132,7 +142,7 @@ claims (Unverified _ c _ _) = c
 claims (Verified _ c _) = c
 
 -- | Extract the header from a JSON Web Token
-header :: JWT r -> JWTHeader
+header :: JWT r -> JOSEHeader
 header (Unverified h _ _ _) = h
 header (Verified h _ _) = h
 
@@ -143,11 +153,17 @@ signature (Verified _ _ s) = Just s
 
 -- | A JSON numeric value representing the number of seconds from
 -- 1970-01-01T0:0:0Z UTC until the specified UTC date/time.
-newtype IntDate = IntDate Integer deriving (Show, Eq, Ord)
+{-# DEPRECATED IntDate "Use NumericDate instead. IntDate will be removed in 1.0" #-}
+type IntDate = NumericDate
+
+-- | A JSON numeric value representing the number of seconds from
+-- 1970-01-01T0:0:0Z UTC until the specified UTC date/time.
+newtype NumericDate = NumericDate Integer deriving (Show, Eq, Ord)
+
 
 -- | Return the seconds since 1970-01-01T0:0:0Z UTC for the given 'IntDate'
-secondsSinceEpoch :: IntDate -> NominalDiffTime
-secondsSinceEpoch (IntDate s) = fromInteger s
+secondsSinceEpoch :: NumericDate -> NominalDiffTime
+secondsSinceEpoch (NumericDate s) = fromInteger s
 
 -- | A JSON string value, with the additional requirement that while
 -- arbitrary string values MAY be used, any value containing a ":"
@@ -160,12 +176,11 @@ instance Show StringOrURI where
     show (S s) = T.unpack s
     show (U u) = show u
 
-
 data Algorithm = HS256 -- ^ HMAC using SHA-256 hash algorithm
                  deriving (Eq, Show)
 
--- | JWT Header, describes the cryptographic operations applied to the JWT
-data JWTHeader = JWTHeader {
+-- | JOSE Header, describes the cryptographic operations applied to the JWT
+data JOSEHeader = JOSEHeader {
     -- | The typ (type) Header Parameter defined by [JWS] and [JWE] is used to
     -- declare the MIME Media Type [IANA.MediaTypes] of this complete JWT in
     -- contexts where this is useful to the application.
@@ -182,8 +197,8 @@ data JWTHeader = JWTHeader {
   , alg :: Maybe Algorithm
 } deriving (Eq, Show)
 
-instance Default JWTHeader where
-    def = JWTHeader Nothing Nothing Nothing
+instance Default JOSEHeader where
+    def = JOSEHeader Nothing Nothing Nothing
 
 -- | The JWT Claims Set represents a JSON object whose members are the claims conveyed by the JWT.
 data JWTClaimsSet = JWTClaimsSet {
@@ -220,11 +235,9 @@ instance Default JWTClaimsSet where
     def = JWTClaimsSet Nothing Nothing Nothing Nothing Nothing Nothing Nothing Map.empty
 
 
-
-
 -- | Encode a claims set using the given secret
 --
--- >>> :{
+--  @
 --  let
 --      cs = def { -- def returns a default JWTClaimsSet
 --         iss = stringOrURI "Foo"
@@ -232,8 +245,8 @@ instance Default JWTClaimsSet where
 --      }
 --      key = secret "secret-key"
 --  in encodeSigned HS256 key cs
--- :}
--- "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJodHRwOi8vZXhhbXBsZS5jb20vaXNfcm9vdCI6dHJ1ZSwiaXNzIjoiRm9vIn0.vHQHuG3ujbnBUmEp-fSUtYxk27rLiP2hrNhxpyWhb2E"
+-- @
+-- > "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJodHRwOi8vZXhhbXBsZS5jb20vaXNfcm9vdCI6dHJ1ZSwiaXNzIjoiRm9vIn0.vHQHuG3ujbnBUmEp-fSUtYxk27rLiP2hrNhxpyWhb2E"
 encodeSigned :: Algorithm -> Secret -> JWTClaimsSet -> JSON
 encodeSigned algo secret claims = dotted [header, claim, signature]
     where claim     = encodeJWT claims
@@ -245,16 +258,16 @@ encodeSigned algo secret claims = dotted [header, claim, signature]
 
 -- | Encode a claims set without signing it
 --
--- >>> :{
+--  @
 --  let
 --      cs = def { -- def returns a default JWTClaimsSet
 --      iss = stringOrURI "Foo"
---    , iat = intDate 1394700934
+--    , iat = numericDate 1394700934
 --    , unregisteredClaims = Map.fromList [("http://example.com/is_root", (Bool True))]
 --  }
 --  in encodeUnsigned cs
--- :}
--- "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpYXQiOjEzOTQ3MDA5MzQsImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlLCJpc3MiOiJGb28ifQ."
+--  @
+-- > "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpYXQiOjEzOTQ3MDA5MzQsImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlLCJpc3MiOiJGb28ifQ."
 encodeUnsigned :: JWTClaimsSet -> JSON
 encodeUnsigned claims = dotted [header, claim, ""]
     where claim     = encodeJWT claims
@@ -263,7 +276,6 @@ encodeUnsigned claims = dotted [header, claim, ""]
                       , alg = Just HS256
                       }
 
-
 -- | Decode a claims set without verifying the signature. This is useful if
 -- information from the claim set is required in order to verify the claim
 -- (e.g. the secret needs to be retrieved based on unverified information
@@ -275,7 +287,7 @@ encodeUnsigned claims = dotted [header, claim, ""]
 --      mJwt = decode input
 --  in fmap header mJwt
 -- :}
--- Just (JWTHeader {typ = Just "JWT", cty = Nothing, alg = Just HS256})
+-- Just (JOSEHeader {typ = Just "JWT", cty = Nothing, alg = Just HS256})
 --
 -- and
 --
@@ -294,7 +306,7 @@ decode input = do
     Unverified <$> header' <*> claims' <*> (pure . Signature $ s) <*> (pure . dotted $ [h,c])
     where
         extractElems (h:c:s:_) = Just (h,c,s)
-        extractElems _       = Nothing
+        extractElems _         = Nothing
 
 -- | Using a known secret and a decoded claims set verify that the signature is correct
 -- and return a verified JWT token as a result.
@@ -343,33 +355,42 @@ decodeAndVerifySignature secret' input = verify secret' =<< decode input
 tokenIssuer :: JSON -> Maybe StringOrURI
 tokenIssuer = decode >=> fmap pure claims >=> iss
 
--- | Create a Secret using the given key
--- This will currently simply wrap the given key appropriately buy may
--- return a Nothing in the future if the key needs to adhere to a specific
--- format and the given key is invalid.
+-- | Create a Secret using the given key.
+-- Consider using `binarySecret` instead if your key is not already a "Data.Text".
 secret :: T.Text -> Secret
-secret = Secret
+secret = Secret . TE.encodeUtf8
+
+-- | Create a Secret using the given key.
+binarySecret :: BS.ByteString -> Secret
+binarySecret = Secret
 
 -- | Convert the `NominalDiffTime` into an IntDate. Returns a Nothing if the
 -- argument is invalid (e.g. the NominalDiffTime must be convertible into a
 -- positive Integer representing the seconds since epoch).
+{-# DEPRECATED intDate "Use numericDate instead. intDate will be removed in 1.0" #-}
 intDate :: NominalDiffTime -> Maybe IntDate
-intDate i | i < 0 = Nothing
-intDate i = Just $ IntDate $ round i
+intDate = numericDate
+
+-- | Convert the `NominalDiffTime` into an NumericDate. Returns a Nothing if the
+-- argument is invalid (e.g. the NominalDiffTime must be convertible into a
+-- positive Integer representing the seconds since epoch).
+numericDate :: NominalDiffTime -> Maybe NumericDate
+numericDate i | i < 0 = Nothing
+numericDate i         = Just $ NumericDate $ round i
 
 -- | Convert a `T.Text` into a 'StringOrURI`. Returns a Nothing if the
 -- String cannot be converted (e.g. if the String contains a ':' but is
 -- *not* a valid URI).
 stringOrURI :: T.Text -> Maybe StringOrURI
 stringOrURI t | URI.isURI $ T.unpack t = U <$> URI.parseURI (T.unpack t)
-stringOrURI t = Just (S t)
+stringOrURI t                          = Just (S t)
 
 
 -- | Convert a `StringOrURI` into a `T.Text`. Returns the T.Text
 -- representing the String as-is or a Text representation of the URI
 -- otherwise.
 stringOrURIToText :: StringOrURI -> T.Text
-stringOrURIToText (S t) = t
+stringOrURIToText (S t)   = t
 stringOrURIToText (U uri) = T.pack $ URI.uriToString id uri (""::String)
 
 -- | Convert the `aud` claim in a `JWTClaimsSet` into a `[StringOrURI]`
@@ -382,10 +403,12 @@ auds jwt = case aud jwt of
 -- =================================================================================
 
 encodeJWT :: ToJSON a => a -> T.Text
-encodeJWT = base64Encode . TE.decodeUtf8 . BL.toStrict . JSON.encode
+encodeJWT = TE.decodeUtf8 . convertToBase Base64URLUnpadded . BL.toStrict . JSON.encode
 
 parseJWT :: FromJSON a => T.Text -> Maybe a
-parseJWT = JSON.decode . BL.fromStrict . TE.encodeUtf8 . base64Decode
+parseJWT x = case convertFromBase Base64URLUnpadded $ TE.encodeUtf8 x of
+               Left _  -> Nothing
+               Right s -> JSON.decode $ BL.fromStrict s
 
 dotted :: [T.Text] -> T.Text
 dotted = T.intercalate "."
@@ -394,8 +417,9 @@ dotted = T.intercalate "."
 -- =================================================================================
 
 calculateDigest :: Algorithm -> Secret -> T.Text -> T.Text
-calculateDigest HS256 (Secret key) msg = base64Encode' $ HMAC.hmac SHA.hash 64 (bs key) (bs msg)
-    where bs = TE.encodeUtf8
+calculateDigest HS256 (Secret key) msg = TE.decodeUtf8 $ convertToBase Base64URLUnpadded (hmac key (bs msg) :: HMAC SHA256)
+    where 
+        bs = TE.encodeUtf8
 
 -- =================================================================================
 
@@ -406,7 +430,8 @@ fromHashMap = Map.fromList . StrictMap.toList
 
 removeRegisteredClaims :: ClaimsMap -> ClaimsMap
 removeRegisteredClaims input = Map.differenceWithKey (\_ _ _ -> Nothing) input registeredClaims
-    where registeredClaims = Map.fromList $ map (\e -> (e, Null)) ["iss", "sub", "aud", "exp", "nbf", "iat", "jti"]
+    where 
+        registeredClaims = Map.fromList $ map (\e -> (e, Null)) ["iss", "sub", "aud", "exp", "nbf", "iat", "jti"]
 
 instance ToJSON JWTClaimsSet where
     toJSON JWTClaimsSet{..} = object $ catMaybes [
@@ -419,7 +444,6 @@ instance ToJSON JWTClaimsSet where
                 , fmap ("jti" .=) jti
             ] ++ Map.toList (removeRegisteredClaims unregisteredClaims)
 
-
 instance FromJSON JWTClaimsSet where
         parseJSON = withObject "JWTClaimsSet"
                      (\o -> JWTClaimsSet
@@ -435,26 +459,25 @@ instance FromJSON JWTClaimsSet where
                      <*> o .:? "jti"
                      <*> pure (removeRegisteredClaims $ fromHashMap o))
 
-
-instance FromJSON JWTHeader where
-    parseJSON = withObject "JWTHeader"
-                    (\o -> JWTHeader
+instance FromJSON JOSEHeader where
+    parseJSON = withObject "JOSEHeader"
+                    (\o -> JOSEHeader
                     <$> o .:? "typ"
                     <*> o .:? "cty"
                     <*> o .:? "alg")
 
-instance ToJSON JWTHeader where
-    toJSON JWTHeader{..} = object $ catMaybes [
+instance ToJSON JOSEHeader where
+    toJSON JOSEHeader{..} = object $ catMaybes [
                   fmap ("typ" .=) typ
                 , fmap ("cty" .=) cty
                 , fmap ("alg" .=) alg
             ]
 
-instance ToJSON IntDate where
-    toJSON (IntDate i) = Number $ scientific (fromIntegral i) 0
+instance ToJSON NumericDate where
+    toJSON (NumericDate i) = Number $ scientific (fromIntegral i) 0
 
-instance FromJSON IntDate where
-    parseJSON (Number x) = return $ IntDate $ coefficient x
+instance FromJSON NumericDate where
+    parseJSON (Number x) = return $ NumericDate $ coefficient x
     parseJSON _          = mzero
 
 instance ToJSON Algorithm where
@@ -465,13 +488,13 @@ instance FromJSON Algorithm where
     parseJSON _                = mzero
 
 instance ToJSON StringOrURI where
-    toJSON (S s) = String s
+    toJSON (S s)   = String s
     toJSON (U uri) = String $ T.pack $ URI.uriToString id uri ""
 
 instance FromJSON StringOrURI where
     parseJSON (String s) | URI.isURI $ T.unpack s = return $ U $ fromMaybe URI.nullURI $ URI.parseURI $ T.unpack s
-    parseJSON (String s) = return $ S s
-    parseJSON _          = mzero
+    parseJSON (String s)                          = return $ S s
+    parseJSON _                                   = mzero
 
 -- $docDecoding
 -- There are three use cases supported by the set of decoding/verification
